﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Server
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        #region 启动监听
        //启动监听
        private void btnStart_Click(object sender, EventArgs e)
        {
            IPAddress ip = IPAddress.Parse(txtServer.Text);
            IPEndPoint point = new IPEndPoint(ip, Convert.ToInt32(txtPort.Text));
            Socket socketWatch = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);//监听套接字

            //绑定端口 开始监听
            socketWatch.Bind(point);
            //设置排队队列长度
            socketWatch.Listen(10);
            ShowMsg("服务端监听成功...");

            //创建线程用于循环等待用户连接
            Thread th = new Thread(Listen);
            th.IsBackground = true;
            th.Start(socketWatch);
        }
        #endregion

        Dictionary<string, Socket> dicSocket = new Dictionary<string, Socket>();

        #region 监听  void Listen
        //  监听
        void Listen(object o)
        {
            Socket socketWatch = o as Socket;
            while (true)
            {
                Socket socketConnect = socketWatch.Accept();//等待客户端连接
                //添加到键值对集合
                dicSocket[socketConnect.RemoteEndPoint.ToString()] = socketConnect;
                //加到列表
                lbUsers.Items.Add(socketConnect.RemoteEndPoint.ToString());
                //在线用户增加
                lblNum.Text = lbUsers.Items.Count.ToString();
                ShowMsg("客户端（" + socketConnect.RemoteEndPoint.ToString() + "）:上线了...");

                //创建线程用于循环接收
                Thread th2 = new Thread(Rec);
                th2.IsBackground = true;
                th2.Start(socketConnect);
            }
        }
        #endregion

        //循环接收
        void Rec(object o)
        {
            Socket socketConnect = o as Socket;
            //循环接收
            while (true)
            {
                try
                {
                    byte[] buffer = new byte[1024 * 1024];
                    //实际接收到的字节数
                    int r = socketConnect.Receive(buffer);
                    //如果r不大于0
                    if (r <= 0)
                    {
                        break;
                    }
                    if (buffer[0] == 0)//文本
                    {
                        ShowMsg(socketConnect.RemoteEndPoint.ToString() + " : " + System.Text.Encoding.UTF8.GetString(buffer, 1, r - 1));
                    }
                    else if (buffer[0] == 1)//文件
                    {
                        SaveFileDialog sfd = new SaveFileDialog();
                        if (sfd.ShowDialog(this) == DialogResult.OK)
                        {
                            string path = sfd.FileName;
                            using (FileStream fsWrite = new FileStream(path, FileMode.OpenOrCreate, FileAccess.Write))
                            {
                                fsWrite.Write(buffer, 1, r - 1);
                            }
                            ShowMsg("文件接收完成 " + path);
                        }
                    }
                }
                catch (Exception ex)//远程主机关闭连接
                {
                    ShowMsg(socketConnect.RemoteEndPoint.ToString() + ":" + ex.Message);
                    //从集合移除
                    dicSocket.Remove(socketConnect.RemoteEndPoint.ToString());
                    //列表里移除
                    lbUsers.Items.Remove(socketConnect.RemoteEndPoint.ToString());
                    //在线人数减少
                    lblNum.Text = lbUsers.Items.Count.ToString();
                    break;
                }
            }
        }

        //追加消息
        void ShowMsg(string msg)
        {
            txtLog.AppendText(msg + "\r\n");
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Control.CheckForIllegalCrossThreadCalls = false;
        }

        //发送消息
        private void btnSend_Click(object sender, EventArgs e)
        {
            //如果群发
            if (cbAllSend.Checked)
            {
                foreach (Socket kv in dicSocket.Values)
                {
                    SendMsg(kv, txtMsg.Text);
                    Thread.Sleep(100);//休息0.1秒
                }
                ShowMsg("群发完成");
            }
            else
            {
                //如果未选择任何客户端
                if (lbUsers.SelectedItem == null)
                {
                    ShowMsg("系统提示: 请先选择客户端...");
                    return;
                }
                //当前选择的客户端
                Socket socket = dicSocket[lbUsers.SelectedItem.ToString()];
                SendMsg(socket, txtMsg.Text);
            }
        }

        //选择文件
        private void btnSelect_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                txtPath.Text = ofd.FileName;
            }
        }

        //发送文件
        private void btnSendFile_Click(object sender, EventArgs e)
        {
            //如果群发
            if (cbAllSend.Checked)
            {
                foreach (Socket kv in dicSocket.Values)
                {
                    SendFile(kv, txtPath.Text);
                    Thread.Sleep(100);//休息0.1秒
                }
                ShowMsg("群发完成");
            }
            else
            {
                //如果未选择任何客户端
                if (lbUsers.SelectedItem == null)
                {
                    ShowMsg("系统提示: 请先选择客户端...");
                    return;
                }
                //当前选择的客户端
                Socket socket = dicSocket[lbUsers.SelectedItem.ToString()];
                SendFile(socket, txtPath.Text);
            }
        }

        #region 套接字发送消息  void SendMsg
        //套接字发送消息
        void SendMsg(Socket socketConnect, string msg)
        {
            byte[] buffer = System.Text.Encoding.UTF8.GetBytes(msg);

            //byte[] msgSend = new byte[buffer.Length + 1];
            //msgSend[0] = 0;//消息
            //Buffer.BlockCopy(buffer, 0, msgSend, 1, buffer.Length);//复制到新数组
            List<byte> list = new List<byte>();
            list.Add(0);//消息
            list.AddRange(buffer);
            socketConnect.Send(list.ToArray());
        }
        #endregion

        #region 向套接字发送指定文件 void SendFile
        //向套接字发送指定文件
        void SendFile(Socket socketConnect, string path)
        {
            //文件流读文件
            using (FileStream fsRead = new FileStream(path, FileMode.Open, FileAccess.Read))
            {
                string fileName = path.Substring(path.LastIndexOf("\\") + 1);//截取文件名
                ShowMsg(fileName);
                byte[] buffer = new byte[1024 * 1024 * 5];
                //while (true)
                //{
                int r = fsRead.Read(buffer, 0, buffer.Length);
                //if (r <= 0)
                //{
                //    break;
                //}
                byte[] bufferSend = new byte[r + 51];//新建数组
                bufferSend[0] = 1;//文件
                //文件名
                for (int i = 0; i < fileName.Length; i++)
                {
                    bufferSend[i + 1] = Convert.ToByte(fileName[i]);
                }
                Buffer.BlockCopy(buffer, 0, bufferSend, 51, r);//复制到新数组
                socketConnect.Send(bufferSend);//发送文件
                //}    
            }
            ShowMsg("文件发送完成");
        }
        #endregion

        #region 震动 private void btnZD_Click
        //震动
        private void btnZD_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[1];
            buffer[0] = 2;//震动
            //如果群发
            if (cbAllSend.Checked)
            {
                foreach (Socket kv in dicSocket.Values)
                {
                    kv.Send(buffer);
                    Thread.Sleep(100);//休息0.1秒
                }
                ShowMsg("群震完成");
            }
            else
            {
                //如果未选择任何客户端
                if (lbUsers.SelectedItem == null)
                {
                    ShowMsg("系统提示: 请先选择客户端...");
                    return;
                }
                //当前选择的客户端
                Socket socket = dicSocket[lbUsers.SelectedItem.ToString()];
                socket.Send(buffer);
            }
        }

        #endregion
    }
}